from .. import Module, Log, Settings
from .. import ConfigFileNotFound
from .. import SubscriberUserNotice, GiftedSubscriberUserNotice


log = Log("CORE:Subs", verbose="log")

class Subs(Module):

    def __init__(self, channel):
        Module.__init__(self, "Subs")
        self.channel = channel
        self.log = Log("Subs")
        self.giftedSubCount = 0
        self.giftedSubCountOffset = 0
        self.giftedSubNoticeSent = True
        self.giftedSubs = []


    def on_subscriber_usernotice(self, data: SubscriberUserNotice):
        service = data.cumulative_months
        
        if service == 0:
            service = 1
        
        message = ""
        if service == 1:
            message = f"{data.display_name} just subscribed for the first time! Welcome to the family ♥"
        else:
            month = service % 12
            if month > 1: mt = "s"
            years = int(service/12)
            if years > 0: 
                if years > 1: yt = f"{years} years and " 
                else: yt = "1 year and "  
            message = f"{data.display_name} resubscribed for another month. That's a total of {yt} {month} month{mt}"
        self.channel.send(message)


    def on_giftedsubscriber_usernotice(self, data: GiftedSubscriberUserNotice):
        if data.anonymous: name = "An anonymous legend"
        else: name = data.sender_display_name
        if data.mass_gift_count > 0:
            self.giftedSubCount = data.mass_gift_count
            self.giftedSubCountOffset = 1
            self.giftedSubNoticeSent = False
            self.giftedSubs = []
        else:
            if self.giftedSubNoticeSent:
                message = f"{name} gifted a sub to {data.recipient_display_name}"
                self.channel.send(message)
            else:
                if self.giftedSubCount <= 10:
                    if (self.giftedSubCount - self.giftedSubCountOffset) == 1:
                        names = ""
                        message = f"{name} gifted {self.giftedSubCount} subs to"
                        for name in self.giftedSubs:
                            names += f" {name},"
                        message += names[:-1] + f" and {data.recipient_display_name} KAPOW"
                        self.channel.send(message)
                    else:
                        self.giftedSubCountOffset += 1
                        self.giftedSubs.append(data.recipient_display_name)
                else:
                    if not self.giftedSubNoticeSent:
                        self.giftedSubNoticeSent = True
                        others = self.giftedSubCount - 1
                        message = f"{name} gifted {self.giftedSubCount} subs to {data.recipient_display_name} and {others} others PogChamp"
                        self.channel.send(message)
            


def setup(channel):
    try:
        settings = Settings()
        try:
            if settings.get_setting("commands.subs") == True:
                channel.load_module(Subs(channel))
                log.log(f"[{channel.channel_name}]: Loaded Module Subs")
        except KeyError:
            pass # command.subs not in settings file
    except ConfigFileNotFound:
        pass # settings file not found


def teardown(channel):
    log.log(f"[{channel.channel_name}]: Removed Module Subs")
    


from .channel import Channel, ConnectedChannel
from .httpclient import HTTPClient, WebResponse
from .log import Log
from time import sleep
from datetime import datetime


api_users = "https://api.twitch.tv/helix/users?"

class Bridge():
    """description of class"""
    channels: list
    active: bool
    


    def __init__(self, client):
        self.client = client
        self.active = True
        self.web = HTTPClient("Bridge")
        self.log = Log("CORE")
        self.lastCheck = datetime.now()
        self.__interval = 7
        pass


    
    def __init_channels(self, connected: bool):
        self.channels = []
        chns = []
        for chn in self.__get_channel_shortlist():
            response: WebResponse = self.web.GetFromTwitch(api_users+chn)
            if response.code == 200 or response.code == "200":
                for user_data in response.json["data"]:
                    chns.append({
                        "login": user_data["login"],
                        "id": user_data["id"]
                    })
            else:
                self.log.error(f"There was an issue communicating with Twitch. Details: {response.message}")
        for chan in chns:
            if connected:
                channel = ConnectedChannel(self, self.client.nick, self.client.token, chan["login"], chan["id"])
            else:
                channel = Channel(self, self.client.nick, self.client.token, chan["login"], chan["id"])
            self.channels.append(channel)
            

    def __get_channel_shortlist(self):
        out = ""
        counter = 0
        maxCount = 80
        for chn in self.client.channels:
            if counter < maxCount:
                out += f"login={chn}&"
            else:
                yield out[:-1]
                out = f"login={chn}&"
                counter = 0
        yield out[:-1]


    def __get_channel(self, name: str):
        for channel in self.channels:
            if channel.name == name:
                return channel
        



    def run(self, connected = False):
        self.__init_channels(connected)
        self.lastCheck = datetime.now()
        try:
            for channel in self.channels:
                sleep(2)
                channel.start()
            while self.active:
                sleep(0.0001)
                if (datetime.now() - self.lastCheck).seconds > self.__interval:
                    self.client.check_messages()
                    self.heartbeat()
                    self.lastCheck = datetime.now()

        except KeyboardInterrupt:         
            for channel in self.channels:
                channel.close()

    def heartbeat(self):
        for channel in self.channels:
            channel.on_heartbeat()

    def send_message_to_channel(self, channel: str, message: str):
        for chn in self.channels:
            if chn.channel_name == channel:
                chn.send(message)

    def set_interval(self, interval: int):
        self.__interval = interval
    
    
    def add_channel(self, channel: str, id: int = None):
        if id is None:
            response: WebResponse = self.web.GetFromTwitch(f"{api_users}login={channel}")
            if response.code == 200 or response.code == "200":
                id = response.json["data"][0]["id"]
            else:
                self.log.error(f"Could not Add Channel. There was an issue communicating with Twitch. Details: {response.message}")
                return

        chn = Channel(self, self.client.nick, self.client.token, channel, id=id)
        self.channels.append(chn)
        chn.start()
        chn.on_new_connection(channel)
        self.client.on_add_channel(channel, id)
    
    def remove_channel(self, channel: str):
        for chn in self.channels:
            if chn.channel_name == channel:
                chn.close()
                self.channels.remove(chn)
        self.client.on_remove_channel(channel)





    


